@@ -34,8 +34,12 @@ import spec
 import logger
 from vercmp import FairVersion as _V
 
+USER_CONFIG = '~/.spectacle/spectacle.conf'
+
+# Path for series file that contains patches (and comment lines #)
 SERIES_PATH = 'series.conf'
 
+# Mandatory keys for main package
 MAND_KEYS = ('Name',
              'Summary',
              'Description',
@@ -44,6 +48,7 @@ MAND_KEYS = ('Name',
              'License',
             )
 
+# Mandatory keys for subpackage
 SUB_MAND_KEYS = ('Name',
                  'Summary',
                  'Description',
@@ -63,13 +68,18 @@ BOOLNO_KEYS = ('Check',
                'NoDesktop',
                'UpdateDesktopDB',
                'NoIconCache',
+               'NoSystemdService',
               )
+
 # boolean keys with the default 'True' value
 BOOLYES_KEYS = ('UseAsNeeded',
                 'AutoDepend',
                )
+               
+# All boolean keys
 BOOL_KEYS = BOOLNO_KEYS + BOOLYES_KEYS
 
+# Keys expected to have list type value.
 LIST_KEYS = ('Sources',
              'ExtraSources',
              'Patches',
@@ -90,9 +100,12 @@ LIST_KEYS = ('Sources',
              'Files',
              'Documents',
              'RunFdupes',
-             'RpmLintIgnore'
+             'RpmLintIgnore',
+             'Macros',
+             'Macros2',
              )
 
+# Keys expected to have string value.
 STR_KEYS =  ('Name',
              'Summary',
              'Description',
@@ -102,7 +115,9 @@ STR_KEYS =  ('Name',
              'Group',
              'License',
              'URL',
+             # "Source Code Management". In this context URL to git repository.
              'SCM',
+             # Type of archive that is done when SCM is used.
              'Archive',
              'BuildArch',
              'ExclusiveArch',
@@ -112,19 +127,26 @@ STR_KEYS =  ('Name',
              'SetupOptions',
              'LocaleName',
              'LocaleOptions',
+             # Defines the package name where the *.lang file is added.
+             'LocaleFilesPkgName',
              'FilesInput',
              'PostScripts',
              'Prefix',
             )
 
+# Keys that are only available for subpackages
 SUBONLY_KEYS = ('AsWholeName',
                 'AutoDepend',
                 )
 
+# Key that are warned about moving to main package
+# if found from subpackages.
 SUBWARN_KEYS = ('PkgBR',
                 'PkgConfigBR',
                 'BuildConflicts',
                )
+
+# Keys available for subpackages.
 SUBAVAIL_KEYS = ('Name',
                  'Summary',
                  'Description',
@@ -145,37 +167,99 @@ SUBAVAIL_KEYS = ('Name',
                  'NoAutoReqProv',
                  'NoIconCache',
                  'FilesInput',
-                 'Version', 'Release', 'Epoch', 'URL', 'BuildArch' # very rare
+                 # Very rare keys in sub packages
+                 'Version', 
+                 'Release', 
+                 'Epoch', 
+                 'URL', 
+                 'BuildArch',
                 )
 
+# Deprecated keys that are not used anymore and 
+# should be removed from .yaml
 DROP_KEYS = ('PostScripts',
              'Documents',
             )
 
+# Renamed keys.
 RENAMED_KEYS = {'NeedCheckSection': 'Check',
                 'NoLocale': 'NoAutoLocale',
                }
 
+# Common typos in keys
 TYPO_KEYS = {'BuildRequires': 'PkgBR or PkgConfigBR',
              'Url': 'URL',
             }
 
+# Keys that may have archictecture qualifier in front of them.
 ARCHED_KEYS = ('Requires',
                'PkgBR',
                'PkgConfigBR',
                'Patches',
                'ConfigOptions',
                'QMakeOptions',
+               'Files',
               )
+
+# Available architecture qualifiers
 ARCHS = {'ix86': '%{ix86}',
          'arm': '%{arm}',
          'armv5': 'armv5el armv5tel armv5tejl',
          'armv7': 'armv7el armv7tel armv7l armv7hl armv7nhl',
         }
 
+# Different options for "Configure" yaml key.
 CONFIGURES = ('configure', 'reconfigure', 'autogen', 'cmake', 'none')
+
+# Different options for "Builder" yaml key.
 BUILDERS = ('make', 'single-make', 'python', 'perl', 'qmake', 'cmake', 'none')
 
+# Paths that should be replaced with macros when seen in %files.
+# NOTE: Order of this list matters!
+PATHMACROS = (('/usr/bin',     '%{_bindir}'),
+              ('/usr/sbin',    '%{_sbindir}'),
+              ('/usr/lib',     '%{_libdir}'),
+              ('/usr/libexec', '%{_libexecdir}'),
+              ('/usr/include', '%{_includedir}'),
+              ('/usr/share/info',  '%{_infodir}'),
+              ('/usr/share/man',   '%{_mandir}'),
+              ('/usr/share',   '%{_datadir}'),
+              ('/usr',         '%{_prefix}'),
+              ('/etc/rc.d/init.d',  '%{_initddir}'),
+              ('/etc/init.d',  '%{_initddir}'),
+              ('/etc',         '%{_sysconfdir}'),
+              ('/var/lib',     '%{_sharedstatedir}'),
+              ('/var',         '%{_localstatedir}'),
+             )
+
+# Function to read config file to parser
+def read_config_file(configfile,config_parser = None):
+    from ConfigParser import SafeConfigParser
+    configfile = os.path.expanduser(configfile)
+
+    if not os.path.exists(configfile):
+        return None
+    
+    if not config_parser:
+        config_parser = SafeConfigParser()
+    
+    config_parser.read(configfile)
+    
+    return config_parser
+
+# Function to get values from config
+def get_config(configs, gettype, key, section = 'Main'):
+    if not configs:
+        return None
+    if not configs.has_section(section):
+        return None
+    if not configs.has_option(section, key):
+        return None
+    if gettype == "boolean":
+        return configs.getboolean(section,key)
+    else:
+        return configs.get(section,key)
+
 # global helper functions
 def arch_split(value):
     m = re.match('^(\w+):([^:]+)', value)
@@ -214,9 +298,12 @@ class GitAccess():
 
     def get_toptag(self):
         vers = [_V(tag) for tag in self._gettags()]
-        if vers:
-            vers.sort()
-            return str(vers[-1])
+        # Need to filter ^{} entries out
+        # http://stackoverflow.com/questions/5346060/git-tag-why-this-duplicate-tag-in-remotes
+        filtered_vers = [item for item in vers if not str(item).endswith("^{}")]
+        if filtered_vers:
+            filtered_vers.sort()
+            return str(filtered_vers[-1])
 
         return None
 
@@ -238,10 +325,41 @@ class RPMWriter():
                         'HasStatic': False,
                         'Icon': False,
                         'Service': False,
+                        'SystemdService': False,
+                        'SystemdServices': [],
                         'Info': False,
                         'Infos': [],
                     }
 
+    # Templates for AutoSubPackages key
+    asp_templates = {
+                        'devel': { 'Name': "devel",
+                                   'Description': "Development files for %{name}.",
+                                   'Summary': "Development files for %{name}",
+                                   'Group': "Development/Libraries",
+                                   'AutoDepend': "True",
+                                  },
+                        'docs': { 'Name': "docs",
+                                   'Description': "Documentation files for %{name}.",
+                                   'Summary': "Documentation files for %{name}",
+                                   'Group': "Development/Libraries",
+                                   'AutoDepend': "True",
+                                },
+                        'lang': { 'Name': "lang",
+                                   'Description': "Translation files for %{name}.",
+                                   'Summary': "Translation files for %{name}",
+                                   'Group': "Development/Libraries",
+                                   'AutoDepend': "True",
+                                 },
+                        # Used when package doesn't match any of the above.
+                        'unknown': { 'Name': "unknown",
+                                   'Description': "Files for %{name}.",
+                                   'Summary': "Files for %{name}",
+                                   'Group': "Development/Libraries",
+                                   'AutoDepend': "True",
+                                   },
+                    }
+
     def __init__(self, yaml_fpath, spec_fpath=None, clean_old=False, download_new=True, skip_scm=False):
         self.yaml_fpath = yaml_fpath
         now = datetime.datetime.now()
@@ -270,6 +388,8 @@ class RPMWriter():
             self.stream = file(yaml_fpath, 'r')
         except IOError:
             logger.error('Cannot read file: %s' % yaml_fpath)
+        
+        self.configs = read_config_file(USER_CONFIG)
 
     def dump(self):
         # debugging
@@ -428,7 +548,7 @@ class RPMWriter():
             # whether the invalid keys are common typo
             for key in keys[:]:
                 if key in TYPO_KEYS:
-                    logger.warning('"%s" might be a typo of %s, please correct it' %(key,TYPO_KEYS[key]))
+                    logger.warning('"%s" might be a typo of %s, please fix it' %(key,TYPO_KEYS[key]))
                     keys.remove(key)
 
             return keys
@@ -447,7 +567,7 @@ class RPMWriter():
                             warn = False
                             break
                 except IOError:
-                    logger.error('Cannot open "/usr/share/spectacle/GROUPS", maybe the pkg was not installed correctly.')
+                    logger.error('Cannot open "/usr/share/spectacle/GROUPS", maybe the package was not installed correctly.')
 
                 if warn:
                     logger.warning('Group \'%s\' is not in the list of approved groups. See /usr/share/spectacle/GROUPS for the complete list.' % (metadata['Group']))
@@ -467,7 +587,7 @@ class RPMWriter():
             try:
                 pkgcfg = csv.reader(open('/usr/share/spectacle/pkgconfig-provides.csv'), delimiter=',')
             except IOError:
-                logger.error('Cannot open "/usr/share/spectacle/pkgconfig-provides.csv", maybe the pkg was not installed correctly.')
+                logger.error('Cannot open "/usr/share/spectacle/pkgconfig-provides.csv", maybe the package was not installed correctly.')
 
             for row in pkgcfg:
                 pc = re.search('pkgconfig\(([^)]+)\)', row[1])
@@ -488,10 +608,26 @@ class RPMWriter():
             return True
 
         def _check_listkey(metadata, key):
-            """ sub-routine for LIST typed keys checking """
+            """ sub-routine for LIST typed keys checking
+                and will remove all empty and None values
+            """
             if key in metadata and not isinstance(metadata[key], list):
                 return False
-            return True
+
+            if key not in metadata:
+                return True
+
+            try:
+                while True:
+                    metadata[key].remove(None)
+            except ValueError:
+                pass
+
+            try:
+                while True:
+                    metadata[key].remove('')
+            except ValueError:
+                return True
 
         def _check_strkey(metadata, key):
             """ sub-routine for STR typed keys checking """
@@ -561,7 +697,7 @@ class RPMWriter():
 
         def _check_key_nofiles(metadata):
             if 'Files' in metadata:
-                logger.error('both "NoFiles" and "Files" exists in YAML, please correct it')
+                logger.error('both "NoFiles" and "Files" exist in YAML file, please fix it')
             for req in ('Requires',
                         'RequiresPre',
                         'RequiresPreUn',
@@ -770,13 +906,15 @@ PkgBR:
 
         if "Archive" in self.metadata:
             archive = self.metadata['Archive']
-            if archive not in ('bzip2', 'gzip'):
+            if archive not in ('bzip2', 'gzip', 'xz'):
                 archive = 'bzip2'
         else:
             archive = 'bzip2'
 
         if archive == 'bzip2':
             appendix = 'bz2'
+        elif archive == 'xz':
+            appendix = 'xz'
         else:
             appendix = 'gz'
 
@@ -825,7 +963,9 @@ PkgBR:
                 target = s.replace('%{name}', pkg)
                 target = target.replace('%{version}', rev)
                 f_name = os.path.basename(target)
-                if not os.path.isfile(f_name):
+                if os.path.isdir(f_name):
+                    logger.warning('Source file \'%s\' can not be downloaded, because directory with same name already exists.' % (f_name))
+                elif not os.path.isfile(f_name):
                     answer = logger.ask('Need to download source package: %s ?' % f_name)
                     if not answer: break
 
@@ -837,7 +977,10 @@ PkgBR:
                         urlgrabber.urlgrab(target, f_name, progress_obj = text_progress_meter())
                     except urlgrabber.grabber.URLGrabError, e:
                         if e.errno == 14: # HTTPError
-                            logger.warning('Invalid source URL')
+                            logger.warning('Invalid source URL \'%s\'' % (target))
+                            # In error case the file most probably would not be valid,
+                            # thus we remove the invalid file.
+                            os.remove(f_name)
                         else:
                             raise e
                     except KeyboardInterrupt:
@@ -956,12 +1099,13 @@ PkgBR:
                 'Lib': {'RequiresPost': ['/sbin/ldconfig'],
                         'RequiresPostUn': ['/sbin/ldconfig'],
                        },
-                'Icon': {'RequiresPost': ['/bin/touch', 'gtk2'],
+                'Icon': {'RequiresPost': ['/bin/touch', '%{_bindir}/gtk-update-icon-cache'],
                         },
+                 # BuildRequires doesn't support paths, thus this is mentioned as package.
                 'Desktop': {'PkgBR': ['desktop-file-utils'],
                            },
-                'DesktopDB': {'RequiresPost': ['desktop-file-utils'],
-                              'RequiresPostUn': ['desktop-file-utils'],
+                'DesktopDB': {'RequiresPost': ['%{_bindir}/update-desktop-database'],
+                              'RequiresPostUn': ['%{_bindir}/update-desktop-database'],
                              },
                 'Info': {'RequiresPost': ['/sbin/install-info'],
                          'RequiresPostUn': ['/sbin/install-info'],
@@ -969,9 +1113,14 @@ PkgBR:
                 'Service': {'RequiresPost': ['/sbin/service', '/sbin/chkconfig'],
                             'RequiresPostUn': ['/sbin/service', '/sbin/chkconfig'],
                            },
-                'Schema': {'RequiresPost': ['GConf2'],
-                           'RequiresPreUn': ['GConf2'],
-                           'RequiresPre': ['GConf2'],
+                'SystemdService': {'RequiresPost': ['systemd'],
+                               'RequiresPreUn': ['systemd'],
+                               'RequiresPostUn': ['systemd'],
+                               'Requires': ['systemd'],
+                              },
+                'Schema': {'RequiresPost': ['%{_bindir}/gconftool-2'],
+                           'RequiresPreUn': ['%{_bindir}/gconftool-2'],
+                           'RequiresPre': ['%{_bindir}/gconftool-2'],
                           },
         }
 
@@ -1078,6 +1227,26 @@ PkgBR:
             if 'Sources' in self.metadata:
                 self._download_sources()
 
+        if "Macros" in self.metadata:
+            macros_parsed = {}
+            for macro in self.metadata['Macros']:
+                try:
+                    macro_name, macro_value = map(str.strip, macro.split(';'))
+                except:
+                    logger.error('Invalid Macros entry "%s", should be "name;value"' % macro)
+                macros_parsed[macro_name] = macro_value
+            self.metadata['Macros'] = macros_parsed
+
+        if "Macros2" in self.metadata:
+            macros_parsed = {}
+            for macro in self.metadata['Macros2']:
+                try:
+                    macro_name, macro_value = map(str.strip, macro.split(';'))
+                except:
+                    logger.error('Invalid Macros2 entry "%s", should be "name;value"' % macro)
+                macros_parsed[macro_name] = macro_value
+            self.metadata['Macros2'] = macros_parsed
+
         # handle patches with extra options
         if "Patches" in self.metadata:
             patches = self.metadata['Patches']
@@ -1138,8 +1307,20 @@ PkgBR:
             for sp in self.metadata["SubPackages"]:
                 self.extra['subpkgs'][sp['Name']] = copy.deepcopy(self.extra_per_pkg)
         if "AutoSubPackages" in self.metadata:
+            if not self.metadata.has_key('SubPackages'):
+                self.metadata['SubPackages'] = []
             for asp in self.metadata["AutoSubPackages"]:
                 self.extra['subpkgs'][asp] = copy.deepcopy(self.extra_per_pkg)
+                if self.asp_templates.has_key(asp):
+                    self.metadata['SubPackages'].append(self.asp_templates[asp])
+                    # By default if the LocaleFilesPkgName isn't defined and we have
+                    # lang subpackage the .lang file is assigned to lang package.
+                    if asp == "lang" and not self.metadata.has_key('LocaleFilesPkgName'):
+                        self.metadata['LocaleFilesPkgName'] = asp
+                else:
+                    unknown_asp_tmp = self.asp_templates['unknown']
+                    unknown_asp_tmp['Name'] = asp
+                    self.metadata['SubPackages'].append(unknown_asp_tmp)
 
         # detect the using UI widget, QT or Gtk2
         all_pkgbr = []
@@ -1151,6 +1332,15 @@ PkgBR:
             if br.startswith('Qt'):
                 self.uiwidget = 'Qt'
 
+        if 'LocaleName' not in self.metadata and 'NoAutoLocale' not in self.metadata:
+            # If LocaleName or NoAutoLocale isn't set lets search if there is 'intltool' build requirement 
+            # and set LocaleName to Name.
+            if self.metadata.has_key("PkgBR"):
+                for br in self.metadata['PkgBR']:
+                    if br == 'intltool':
+                        self.metadata['LocaleName'] = self.metadata['Name']
+                        break
+                
         """ NOTE
         we need NOT to do the following checking:
          * whether auto-added Requires(include pre/post/preun/postun) duplicated
@@ -1162,14 +1352,18 @@ PkgBR:
         if pkgname == 'main':
             return self.metadata
 
-        for sp in self.metadata['SubPackages']:
-            if sp['Name'] == pkgname:
-                return sp
+        try:
+            for sp in self.metadata['SubPackages']:
+                if sp['Name'] == pkgname:
+                    return sp
+        except KeyError:
+            # Not a available subpackage for 'pkgname'
+            return {}
 
         # not found
         return {}
 
-    def parse_files(self, files = {}, docs = {}):
+    def parse_files(self, files):
 
         py_path_check = False
         if 'Builder' in self.metadata and self.metadata['Builder'] == 'python':
@@ -1188,6 +1382,13 @@ PkgBR:
                 pkg_extra = self.extra['subpkgs'][pkg_name]
 
             for l in v:
+                # check and warn improper file path
+                for prefix, macro in PATHMACROS:
+                    if l.startswith(prefix):
+                        logger.warning('for %%files line: "%s"\n' % (l,) +\
+                                '\tplease use %s to replace the leading path %s' % (macro, prefix))
+                        break
+
                 if re.match('\s*%exclude\s.*', l):
                     pass # not match anyting excluded files
                 elif re.match('.*\.info\..*', l) or re.match('.*(usr/share/info|%{_infodir}).*info\..*$', l):
@@ -1195,29 +1396,44 @@ PkgBR:
                     l1 = p1.sub(r'\1', l)
                     pkg_extra['Infos'].append(l1)
                     pkg_extra['Info'] = True
+
                 elif re.match('.*(usr/share|%{_datadir})/applications/.*\.desktop$', l):
                     if 'NoDesktop' not in self.metadata:
-                        pkg_extra['Desktop'] = True
+                        # any pkg (main and every sub-pkg) will affect global settings
+                        self.extra['Desktop'] = True
+
                 elif re.match('.*(/etc|%{_sysconfdir})/rc.d/init.d/.*', l) or \
                      re.match('.*(/etc|%{_sysconfdir})/init.d/.*', l) or \
                      re.match('.*%{_initddir}/.*', l) or \
                      re.match('.*%{_initrddir}/.*', l):
-
+                    # legacy init scripts
                     pkg_extra['Service'] = True
+
+                elif re.match('^/(lib|%{_lib})/systemd/system/[^/*]*.service$', l):
+                    # new service for systemd
+                    if 'NoSystemdService' not in self.metadata:
+                        pkg_extra['SystemdService'] = True
+                        service = l.split('systemd/system/')[-1]
+                        if service not in pkg_extra['SystemdServices']:
+                            pkg_extra['SystemdServices'].append(service)
+
                 elif re.match('.*(%{_libdir}|%{_lib}|/lib|/usr/lib)/[^/]*[.*?]+so([.*?]+.*$|$)', l) or \
                    re.match('.*(/ld.so.conf.d/).*', l):
                     if pkg_name != 'devel' and not pkg_name.endswith('-devel'):
                         # 'devel' sub pkgs should not set Lib flags
                         pkg_extra['Lib'] = True
+
                 elif re.match('.*(%{_libdir}|%{_lib}).*', l) and re.match('.*\.a$', l):
                     # if *.a found, set 'HasStatic' flag for MAIN pkg
                     self.extra['HasStatic'] = True
+
                 elif re.match('.*\.schema.*', l):
                     comp = l.split()
                     if len(comp) > 1:
                         l = comp[1]
                     pkg_extra['Schema'] = True
                     pkg_extra['Schemas'].append(l)
+
                 elif re.match('.*\/icons\/.*', l):
                     if 'NoIconCache' in pkg_meta and pkg_meta['NoIconCache'] == True:
                         # using "NoIconCache" to avoid cache explicitly
@@ -1241,21 +1457,6 @@ PkgBR:
             if self.extra['Desktop'] != True:
                 logger.warning('"UpdateDesktopDB" specified but found no desktop files')
 
-        # files listed in '%doc' need handling
-        # TODO to be cleanup
-        for pkg_name,v in docs.iteritems():
-            if pkg_name == 'main':
-                pkg_extra = self.extra
-            else:
-                pkg_extra = self.extra['subpkgs'][pkg_name]
-
-            for l in v:
-                for item in l.split():
-                    if re.match('.*\.info.*', item) or \
-                       re.match('.*(usr/share/info|%{_infodir}).*', item):
-                        pkg_extra['Info'] = True
-                        pkg_extra['Infos'].append(item)
-
     def parse_existing(self, spec_fpath):
         sin = re.compile("^# >> ([^\s]+)\s*(.*)")
         sout = re.compile("^# << ([^\s]+)\s*(.*)")
@@ -1269,91 +1470,147 @@ PkgBR:
         files = {}
         install = {}
         build = {}
-        macros = {}         # global macros
+        macros = {}         # macros added after Name: field in .spec
+        macros2 = {}        # macros added before %prep section in .spec
         setup = {}
         pre = {}
         preun = {}
         post = {}
         postun = {}
-        check = {} # extra headers
+        check = {}          # extra headers
 
         line_num = 0
         for i in file(spec_fpath):
+            line_num += 1
             i = i.strip()
-            if line_num < 3:
-                if line_num == 2:
+            if line_num < 4:
+                if line_num == 3:
                     m = re.match("^#.*spectacle version (\S+)$", i)
                     if m:
                         version = m.group(1)
                         spec_ver = _V(version)
                         cur_ver = _V(__version__.VERSION)
                         if cur_ver < spec_ver:
-                            logger.warning('!!! Current spectacle version is lower than the one used for this package last time')
-                            answer = logger.ask('Please upgrade your spectacle firstly, continue?', False)
+                            logger.warning('!!! Current spectacle version is lower than the one used for this package previously')
+                            default_answer = False
+                            if self.configs:
+                                old_spectacle_ok = get_config(self.configs,'boolean','old_spectacle_ok')
+                                if old_spectacle_ok is not None:
+                                    default_answer = old_spectacle_ok
+                            answer = logger.ask('Please upgrade your spectacle, continue?', default_answer)
                             if not answer:
                                 sys.exit(1)
                     else:
-                        answer = logger.ask('The exist spec file might be not a spectacle generated one, continue?', False)
+                        answer = logger.ask('Existing spec file was not generated by spectacle, continue?', False)
                         if not answer:
                             sys.exit(1)
-                line_num += 1
 
             matchin = sin.match(i)
             matchout = sout.match(i)
 
-            if matchin and not record:
+            if matchin:
+                if record:
+                    logger.error('%s:%d "%s %s" placeholder starting without ending previous %s:%d "%s %s"' % (spec_fpath, line_num, matchin.group(1), matchin.group(2), spec_fpath, start_line_num, ingroup1, ingroup2) )
                 record = True
                 recording = []
+                ingroup1 = matchin.group(1)
+                ingroup2 = matchin.group(2)
+                start_line_num = line_num
                 continue
 
+
             if matchout:
+                if not record:
+                    logger.error('%s:%d "%s %s" placeholder ending whithout starting' % (spec_fpath, line_num, matchout.group(1), matchout.group(2)))
                 record = False
+
+                if matchout.group(1) != ingroup1 or matchout.group(2) != ingroup2 :
+                    logger.error('%s:%d "%s %s" placeholder ending not match starting %s:%d "%s %s"' % (spec_fpath, line_num, matchout.group(1), matchout.group(2),spec_fpath, start_line_num, ingroup1, ingroup2))
+
                 if not recording: continue # empty
 
                 if matchout.group(2) and matchout.group(1) in ["files", "post","postun", "pre", "preun"]:
                     if not matchout.group(2) in self.extra['subpkgs']:
-                        logger.error('In spec %s section for %s package. Package does not excist in YAML.' % (matchout.group(1), matchout.group(2)))
-
+                        logger.error('In %s:%d %s section for lost sub-package: %s. Please fix it and try again.' % (spec_fpath, line_num, matchout.group(1), matchout.group(2)))
                 if matchout.group(1) == "files":
                     if matchout.group(2):
+                        if matchout.group(2) in files:
+                            logger.error('%s:%d two files %s section.' % (spec_fpath, line_num,matchout.group(2)))
                         files[matchout.group(2)] = recording
                     else:
+                        if 'main' in files:
+                            logger.error('%s:%d two files section.' % (spec_fpath, line_num))
                         files['main'] = recording
                 elif matchout.group(1) == "post":
                     if matchout.group(2):
+                        if matchout.group(2) in post:
+                            logger.error('%s:%d two post %s section.' % (spec_fpath, line_num,matchout.group(2)))
                         post[matchout.group(2)] = recording
                     else:
+                        if 'main' in post:
+                            logger.error('%s:%d two post section.' % (spec_fpath, line_num))
                         post['main'] = recording
                 elif matchout.group(1) == "postun":
                     if matchout.group(2):
+                        if matchout.group(2) in postun:
+                            logger.error('%s:%d two postun %s section.' % (spec_fpath, line_num,matchout.group(2)))
                         postun[matchout.group(2)] = recording
                     else:
+                        if 'main' in postun:
+                            logger.error('%s:%d two postun section.' % (spec_fpath, line_num))
                         postun['main'] = recording
                 elif matchout.group(1) == "pre":
                     if matchout.group(2):
+                        if matchout.group(2) in pre:
+                            logger.error('%s:%d two pre %s section.' % (spec_fpath, line_num,matchout.group(2)))
                         pre[matchout.group(2)] = recording
                     else:
+                        if 'main' in pre:
+                            logger.error('%s:%d two pre section.' % (spec_fpath, line_num))
                         pre['main'] = recording
                 elif matchout.group(1) == "preun":
                     if matchout.group(2):
+                        if matchout.group(2) in preun:
+                            logger.error('%s:%d two preun %s section.' % (spec_fpath, line_num,matchout.group(2)))
                         preun[matchout.group(2)] = recording
                     else:
+                        if 'main' in preun:
+                            logger.error('%s:%d two preun section.' % (spec_fpath, line_num))
                         preun['main'] = recording
                 elif matchout.group(1) == "install":
+                    if matchout.group(2) in install:
+                        logger.error('%s:%d two install %s section.' % (spec_fpath, line_num,matchout.group(2)))
                     install[matchout.group(2)] = recording
                 elif matchout.group(1) == "build":
+                    if 'main' in build:
+                        logger.error('%s:%d two build section.' % (spec_fpath, line_num))
                     build[matchout.group(2)] = recording
                 elif matchout.group(1) == "macros":
+                    if 'main' in macros:
+                        logger.error('%s:%d two macros section.' % (spec_fpath, line_num))
                     macros['main'] = recording
+                elif matchout.group(1) == "macros2":
+                    if 'main' in macros2:
+                        logger.error('%s:%d two macros2 section.' % (spec_fpath, line_num))
+                    macros2['main'] = recording
                 elif matchout.group(1) == "setup":
+                    if 'main' in setup:
+                        logger.error('%s:%d two setup section.' % (spec_fpath, line_num))
                     setup['main'] = recording
                 elif matchout.group(1) == "check" or \
                      matchout.group(1) == "check_scriptlets": #TODO, remove it whenever cleanup
+                    if 'main' in check:
+                        logger.error('%s:%d two check section.' % (spec_fpath, line_num))
                     check['main'] = recording
+                else:
+                    logger.error('%s:%d unknown section.' % (spec_fpath, line_num))
 
             if record:
                 recording.append(i)
 
+        if record:
+            logger.error('%s:%d "%s %s" placeholder starting without ending' % (spec_fpath, start_line_num, ingroup1, ingroup2))
+
         content= { "files" : files,
                    "install": install,
                    "build" : build,
@@ -1361,6 +1618,8 @@ PkgBR:
 
         if macros:
             content["macros"] = macros
+        if macros2:
+            content["macros2"] = macros2
         if setup:
             content["setup"] = setup
         if post:
@@ -1387,7 +1646,7 @@ PkgBR:
                         break
 
             if files_yaml:
-                logger.warning('both "Files" YAML keyword and inline %file content in spec present')
+                logger.warning('both "Files" keyword and inline %file content in spec present')
 
         # try to remove duplicate '%defattr' in files list
         for key in content['files']:
@@ -1433,24 +1692,6 @@ PkgBR:
         if extra_content:
             self.extra['content'].update(extra_content)
 
-        """
-        TODO: should not regard them as the content of MAIN pkg
-        if self.extras_filelist:
-            try:
-                self.extra['content']['files']['main'].extend(self.extras_filelist)
-            except KeyError:
-                self.extra['content'].update({'files': {'main': self.extras_filelist}})
-        """
-
-        # TODO, cleanup docs handling when all pkgs need not, include spec.tmpl
-        docs = {}
-        if 'Documents' in self.metadata:
-            docs['main'] = self.metadata['Documents']
-        if "SubPackages" in self.metadata:
-            for sp in self.metadata["SubPackages"]:
-                if 'Documents' in sp:
-                    docs[sp['Name']] = sp['Documents']
-
         if 'files' in self.extra['content']:
             files = copy.deepcopy(self.extra['content']['files'])
         else:
@@ -1469,7 +1710,7 @@ PkgBR:
                     else:
                         files[sp['Name']] = sp['Files']
 
-        self.parse_files(files, docs)
+        self.parse_files(files)
 
         # adding automatic requires according %files
         self._gen_auto_requires(self.metadata, self.extra)
